package com.bettersoft.api.controllers;

import com.bettersoft.api.model.SystemConfigurationBean;
import com.bettersoft.api.service.interfaces.ISystemConfigurationService;
import io.jsonwebtoken.Claims;

import java.io.IOException;
import java.sql.Timestamp;
import java.text.MessageFormat;
import java.util.Date;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.IncorrectCredentialsException;
import org.apache.shiro.authc.UnknownAccountException;
import org.apache.shiro.subject.Subject;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateExpirationFailedException;
import com.bettersoft.api.exceptions.ValidateFailedException;
import com.bettersoft.api.exceptions.ValidateSmsDynamicCodeException;
import com.bettersoft.api.exceptions.ValidateSmsDynamicCodeExpirationException;
import com.bettersoft.api.realm.BettersoftUsernamePasswordToken;
import com.bettersoft.api.service.interfaces.IAuthorizationService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.JwtUtils;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.validategroup.AppLoginForgetPasswordValidate;
import com.bettersoft.api.validategroup.AppLgoinSmsValidate;
import com.bettersoft.api.validategroup.AppLoginPwdValidate;
import com.bettersoft.api.validmodel.LoginAccountValidate;

@RestController
@RequestMapping(value = "/oauth")
public class AuthorizationController extends BaseController {
    @Resource(name = "authorizationService")
    private IAuthorizationService authorizationService;

    @Resource(name = "systemConfigurationService")
    private ISystemConfigurationService service;

    @RequestMapping(value = "/login", method = RequestMethod.POST)
    public void login(HttpServletRequest request, HttpServletResponse response,
                      @Validated LoginAccountValidate loginAccount, BindingResult result)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        String sessionId = !StringUtils.isEmpty(loginAccount.getSessionId()) ? loginAccount
                .getSessionId() : StringUtils.replace(request.getSession()
                .getId(), "-", StringUtils.EMPTY);
        try {
            validateEnityBean(loginAccount, result);

            if (duplicateLogin(loginAccount.getAccount(), sessionId)) {
                jsonResult.setErrorCode("403");
                jsonResult.setSuccess(true);
                return;
            }

            //增加判断是否被锁定的逻辑 20240221 郭
            if(authorizationService.checkIsLock(loginAccount)){
                jsonResult.setErrorCode("403");
                jsonResult.setSuccess(false);
                Integer strLockTime = 30;
                SystemConfigurationBean lockTime = redisCacheUtil
                        .getCache(Constants.SYSCONFIG_KEY, "SYS_INIT_LOCK_TIME");
                if(ObjectUtils.isEmpty(lockTime)){
                    lockTime = service.findConfigurationByCodeNoException("SYS_INIT_LOCK_TIME");
                }
                if(!ObjectUtils.isEmpty(lockTime)){
                    strLockTime = Integer.parseInt(lockTime.getConfigValue());
                }
                jsonResult.setErrMsg("登录失败次数过多，请" + strLockTime + "分钟后重试");
                return;
            }

            Subject subject = SecurityUtils.getSubject();

            BettersoftUsernamePasswordToken token = new BettersoftUsernamePasswordToken(
                    loginAccount.getAccount(), loginAccount.getPassword());
            token.setLoginType(Constants.PCPWD_LOGIN_TYPE);
            token.setLoginPlatform(Constants.PC_PLATFORM);
            token.setSessionId(sessionId);

            // UsernamePasswordToken token = new UsernamePasswordToken(
            // loginAccount.getAccount(), loginAccount.getPassword());
            // token.setHost(Constants.PCPWD_LOGIN_TYPE);

            subject.login(token);

            LoginUserInfo loginUser = (LoginUserInfo) subject.getPrincipal();

            authorizationService.loginUser(loginUser, false);

            jsonResult.setSuccess(true);
            jsonResult.getData().put("token", loginUser.getToken());
            jsonResult.getData().put("refreshToken", loginUser.getRefreshToken());
            jsonResult.getData().put("expireTime", loginUser.getExpireTime().getTime());
            jsonResult.getData().put("sessionId", sessionId);

            authorizationService.resetIsLock(loginAccount);

        } catch (UnknownAccountException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("用户账号或密码不正确！");
        } catch (IncorrectCredentialsException ex) {
            if(ex.getMessage().contains("密码错误")){
                authorizationService.loginError(loginAccount);
            }
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            logger.error("oauth/token error.", ex);
            jsonResult.setSuccess(false);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/loginForce", method = RequestMethod.POST)
    public void loginForce(HttpServletRequest request,
                           HttpServletResponse response,
                           @Validated LoginAccountValidate loginAccount, BindingResult result)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        String sessionId = !StringUtils.isEmpty(loginAccount.getSessionId()) ? loginAccount
                .getSessionId() : StringUtils.replace(request.getSession()
                .getId(), "-", StringUtils.EMPTY);
        try {
            validateEnityBean(loginAccount, result);

            //增加判断是否被锁定的逻辑 20240221 郭
            if(authorizationService.checkIsLock(loginAccount)){
                jsonResult.setErrorCode("403");
                jsonResult.setSuccess(false);
                Integer strLockTime = 30;
                SystemConfigurationBean lockTime = redisCacheUtil
                        .getCache(Constants.SYSCONFIG_KEY, "SYS_INIT_LOCK_TIME");
                if(ObjectUtils.isEmpty(lockTime)){
                    lockTime = service.findConfigurationByCodeNoException("SYS_INIT_LOCK_TIME");
                }
                if(!ObjectUtils.isEmpty(lockTime)){
                    strLockTime = Integer.parseInt(lockTime.getConfigValue());
                }
                jsonResult.setErrMsg("登录失败次数过多，请" + strLockTime + "分钟后重试");
                return;
            }

            Subject subject = SecurityUtils.getSubject();

            BettersoftUsernamePasswordToken token = new BettersoftUsernamePasswordToken(
                    loginAccount.getAccount(), loginAccount.getPassword());
            token.setLoginType(Constants.PCPWD_LOGIN_TYPE);
            token.setLoginPlatform(Constants.PC_PLATFORM);
            token.setSessionId(sessionId);

            subject.login(token);

            LoginUserInfo loginUser = (LoginUserInfo) subject.getPrincipal();

            authorizationService.loginUser(loginUser, true);

            jsonResult.setSuccess(true);
            jsonResult.getData().put("token", loginUser.getToken());
            jsonResult.getData().put("refreshToken", loginUser.getRefreshToken());
            jsonResult.getData().put("expireTime", loginUser.getExpireTime().getTime());
            jsonResult.getData().put("sessionId", sessionId);
            authorizationService.resetIsLock(loginAccount);
        } catch (UnknownAccountException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("用户账号或密码不正确！");
        } catch (IncorrectCredentialsException ex) {
            if(ex.getMessage().contains("密码错误")){
                authorizationService.loginError(loginAccount);
            }
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            logger.error("oauth/token error.", ex);
            jsonResult.setSuccess(false);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/apppwdlogin", method = RequestMethod.POST)
    public void apppwdlogin(
            HttpServletRequest request,
            HttpServletResponse response,
            @Validated(value = {AppLoginPwdValidate.class}) LoginAccountValidate loginAccount,
            BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String sessionId = loginAccount.getSessionId();
        try {
            validateEnityBean(loginAccount, result);
            Subject subject = SecurityUtils.getSubject();

            BettersoftUsernamePasswordToken token = new BettersoftUsernamePasswordToken(
                    loginAccount.getAccount(), loginAccount.getPassword());
            token.setLoginType(Constants.APPPWD_LOGIN_TYPE);
            token.setLoginPlatform(loginAccount.getPlatform());
            token.setSessionId(sessionId);

            subject.login(token);

            LoginUserInfo loginUser = (LoginUserInfo) subject.getPrincipal();

            authorizationService.loginUser(loginUser, false);

            jsonResult.setSuccess(true);
            jsonResult.getData().put("token", loginUser.getToken());
            jsonResult.getData().put("refreshToken", loginUser.getRefreshToken());
            jsonResult.getData().put("expireTime", loginUser.getExpireTime().getTime());
            jsonResult.getData().put("loginUser", loginUser);
        } catch (UnknownAccountException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("不存在此账号！");
        } catch (IncorrectCredentialsException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            logger.error("oauth/apppwdlogin error.", ex);
            jsonResult.setSuccess(false);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/appsmslogin", method = RequestMethod.POST)
    public void appsmslogin(
            HttpServletRequest request,
            HttpServletResponse response,
            @Validated(value = {AppLgoinSmsValidate.class}) LoginAccountValidate loginAccount,
            BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();
        String sessionId = loginAccount.getSessionId();
        try {
            validateEnityBean(loginAccount, result);

            checkSmsCode(loginAccount.getAccount(), loginAccount.getSmsCode());

            Subject subject = SecurityUtils.getSubject();

            BettersoftUsernamePasswordToken token = new BettersoftUsernamePasswordToken(
                    loginAccount.getAccount(), StringUtils.EMPTY);
            token.setLoginType(Constants.APPSMS_LOGIN_TYPE);
            token.setLoginPlatform(loginAccount.getPlatform());
            token.setSessionId(sessionId);
            subject.login(token);

            LoginUserInfo loginUser = (LoginUserInfo) subject.getPrincipal();

            authorizationService.loginUser(loginUser, false);

            jsonResult.setSuccess(true);
            jsonResult.getData().put("token", loginUser.getToken());
            jsonResult.getData().put("refreshToken", loginUser.getRefreshToken());
            jsonResult.getData().put("loginUser", loginUser);
        } catch (UnknownAccountException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrorCode("404");
            jsonResult.setErrMsg("不存在此账号！");
        } catch (IncorrectCredentialsException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (IOException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("短信验证码不允许为空！");
        } catch (ValidateSmsDynamicCodeExpirationException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("短信验证码已过期, 请重新获取！");
        } catch (ValidateSmsDynamicCodeException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("短信验证码输入有误，请重新输入！");
        } catch (Exception ex) {
            logger.error("oauth/appsmslogin error.", ex);
            jsonResult.setSuccess(false);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

//	@RequestMapping(value = "/register", method = RequestMethod.POST)
//	public void register(HttpServletRequest request,
//			HttpServletResponse response,
//			@Validated MembershipValidate registerValidate, BindingResult result)
//			throws Exception {
//		JsonResult jsonResult = new JsonResult();
//
//		try {
//			validateEnityBean(registerValidate, result);
//
//			checkSmsCode(registerValidate.getMobile(),
//					registerValidate.getSmsCode());
//
//			authorizationService.register(registerValidate);
//
//			jsonResult.setSuccess(true);
//		} catch (ValidateSmsDynamicCodeException ex) {
//			jsonResult.setSuccess(false);
//			jsonResult.setErrMsg("无效的短信验证码！");
//		} catch (ValidateSmsDynamicCodeExpirationException ex) {
//			jsonResult.setSuccess(false);
//			jsonResult.setErrMsg("短信验证码已过期，请重新获取！");
//		} catch (ValidateFailedException ex) {
//			jsonResult.setSuccess(false);
//			jsonResult.setErrMsg(ex.getMessage());
//		} catch (ValidateDataFailedException ex) {
//			jsonResult.setSuccess(false);
//			jsonResult.setErrMsg(ex.getMessage());
//		} catch (ValidateDataOccupiedException ex) {
//			jsonResult.setSuccess(false);
//			jsonResult.setErrMsg(MessageFormat.format(
//					"用户登录账号:{0}, 已被注册请重新填写用户登录账号！",
//					registerValidate.getMobile()));
//		} catch (Exception ex) {
//			jsonResult.setSuccess(false);
//			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
//			logger.error("oauth/register error.", ex);
//		} finally {
//			renderResponse(jsonResult, response);
//		}
//	}

    @RequestMapping(value = "/validateToken", method = RequestMethod.POST)
    public void validateToken(HttpServletRequest request,
                              HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            String token = request.getHeader("Authorization");

            logger.info("Auth ticket:" + token);

            if (StringUtils.isEmpty(token))
                throw new ValidateAuthorityFailedException();

            Claims claims = authorizationService.validateLoginUser(token);

            jsonResult.setSuccess(true);
            jsonResult.getData().put("loginUser", claims);
        } catch (ValidateExpirationFailedException ex) {
            jsonResult.getData().put("isExpiration", true);
            throw ex;
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("oauth/validateToken error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/refreshToken/{refreshToken}", method = RequestMethod.POST)
    public void refreshToken(@PathVariable("refreshToken") String refreshToken, HttpServletRequest request,
                             HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {

            LoginUserInfo loginUser = authorizationService.refreshLoginUser(refreshToken);
//            Claims claims = JwtUtils.parseToken(ticket);
//            if (0 > claims.getExpiration().compareTo(new Date())) {
//                LoginUserInfo loginUser = EntityUtil.convertMapToEntity(
//                        LoginUserInfo.class, claims);
//
//                ticket = JwtUtils.createToken(loginUser,
//                        Constants.SESSION_EXPIRE_TIME);
//                redisCacheUtil.setCache(Constants.LOGIN_SESSION_USER
//                                + loginUser.getUserCode(), loginUser.getSessionId(),
//                        loginUser, Constants.SESSION_EXPIRE_TIME);
//            }

            jsonResult.setSuccess(true);
            jsonResult.getData().put("token", loginUser.getToken());
            jsonResult.getData().put("refreshToken", loginUser.getRefreshToken());
            jsonResult.getData().put("expireTime", loginUser.getExpireTime().getTime());
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            throw ex;
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            logger.error("oauth/refreshToken error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/forgetPassword", method = RequestMethod.POST)
    public void forgetPassword(
            HttpServletRequest request,
            HttpServletResponse response,
            @Validated(value = {AppLoginForgetPasswordValidate.class}) LoginAccountValidate accountValidate,
            BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            validateEnityBean(accountValidate, result);

            checkSmsCode(accountValidate.getAccount(),
                    accountValidate.getSmsCode());

            authorizationService.forgetPassword(accountValidate);
            jsonResult.setSuccess(true);
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.getData().put("errMsg", "无法获取用户账号信息！");
        } catch (ValidateSmsDynamicCodeException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("无效的短信验证码！");
        } catch (ValidateSmsDynamicCodeExpirationException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("短信验证码已过期，请重新获取！");
        } catch (ValidateFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.getData().put(
                    "errMsg",
                    MessageFormat.format("手机号{0}，未注册！",
                            accountValidate.getAccount()));
        } catch (Exception ex) {
            logger.error("oauth/forgetPassword error.", ex);
            jsonResult.setSuccess(false);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/logout", method = RequestMethod.POST)
    public void logout(HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            String token = request.getHeader("Authorization");

            Subject subject = SecurityUtils.getSubject();

            if (null != subject) {
                Claims claims = authorizationService.validateLoginUser(token);
                LoginUserInfo loginUser = EntityUtil.convertMapToEntity(
                        LoginUserInfo.class, claims);
                authorizationService.logoutUser(loginUser);
                SecurityUtils.getSecurityManager().logout(subject);
            }

            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            logger.error("oauth/logout error.", ex);
            jsonResult.setSuccess(false);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @SuppressWarnings("rawtypes")
    private boolean duplicateLogin(String account, String sessionId) {
        Map loginMap = redisCacheUtil.getCacheMap(Constants.LOGIN_SESSION_USER
                + account);
        return null != loginMap && !loginMap.containsKey(sessionId);
    }

    @RequestMapping(value = "/loginPwdCheckError", method = RequestMethod.POST)
    public void loginPwdCheckError(HttpServletRequest request, HttpServletResponse response,
                      @Validated LoginAccountValidate loginAccount, BindingResult result)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        try {
            validateEnityBean(loginAccount, result);

            //增加判断是否被锁定的逻辑 20240221 郭
            if(authorizationService.checkIsLock(loginAccount)){
                jsonResult.setErrorCode("403");
                jsonResult.setSuccess(false);
                Integer strLockTime = 30;
                SystemConfigurationBean lockTime = redisCacheUtil
                        .getCache(Constants.SYSCONFIG_KEY, "SYS_INIT_LOCK_TIME");
                if(ObjectUtils.isEmpty(lockTime)){
                    lockTime = service.findConfigurationByCodeNoException("SYS_INIT_LOCK_TIME");
                }
                if(!ObjectUtils.isEmpty(lockTime)){
                    strLockTime = Integer.parseInt(lockTime.getConfigValue());
                }
                jsonResult.setErrMsg("登录失败次数过多，请" + strLockTime + "分钟后重试");
                return;
            }

            authorizationService.loginError(loginAccount);

            jsonResult.setErrMsg("密码不符合校验规则,登录失败");
            jsonResult.setSuccess(false);
            jsonResult.setErrorCode("100");

        } catch (Exception ex) {
            logger.error("oauth/token error.", ex);
            jsonResult.setSuccess(false);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
}
