package com.bettersoft.api.controllers;

import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.ISystemConfigurationService;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.SystemConfigurationValidate;

@RestController
@RequestMapping(value = "/sys/config")
@MenuCode("SYS04")
public class SystemConfigurationController extends BaseController {
	@Resource(name = "systemConfigurationService")
	private ISystemConfigurationService systemConfigurationService;

	@RequestMapping(value = "/{configCode}", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void findConfig(@PathVariable("configCode") String configCode,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		try {
			checkAuthorization(request);

			jsonResult.getData().put(
					"config",
					systemConfigurationService
							.findConfigurationByCode(configCode));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ClassNotFoundException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("无法获取系统配置信息！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("sys/config/findConfig error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchConfigs", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchConfigs(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());
		JsonResult jsonResult = new JsonResult();
		try {
			checkAuthorization(request);

			jsonResult.getData()
					.putAll(systemConfigurationService
							.searchConfigurations(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("sys/config/searchConfigs error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/saveConfig", method = RequestMethod.POST)
	@ValidatePermission("INSERT")
	public void saveConfig(HttpServletRequest request,
			HttpServletResponse response,
			@Validated SystemConfigurationValidate configModel,
			BindingResult result) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);

		try {
			checkAuthorization(request);

			validateEnityBean(configModel, result);
			systemConfigurationService
					.saveConfiguration(configModel, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataOccupiedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统配置编号已存在，不允许重复！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("sys/config/saveConfig error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/updateConfig/{configCode}", method = RequestMethod.POST)
	@ValidatePermission("UPDATE")
	public void updateConfig(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("configCode") String configCode,
			@Validated SystemConfigurationValidate configModel,
			BindingResult result) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try {
			checkAuthorization(request);

			validateEnityBean(configModel, result);

			//修正初始密码的情况下，增加对初始密码的校验
			if(StringUtils.equals(configCode,"SYS_INIT_PWD") && !checkPwdValid(configModel.getConfigValue())){
				jsonResult.setSuccess(false);
				jsonResult.setErrMsg("修改密码失败，新密码不符合校验规则");
			} else {
				systemConfigurationService.updateConfiguration(configCode,
						configModel, loginUser);

				jsonResult.setSuccess(true);
			}
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("sys/dept/updatePerm error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/deleteConfig/{configCode}", method = RequestMethod.POST)
	@ValidatePermission("DELETE")
	public void deletePerm(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("configCode") String configCode) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try {
			checkAuthorization(request);

			systemConfigurationService.deleteConfiguration(configCode,
					loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
			logger.error("sys/config/deleteConfig error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}
}
