package com.bettersoft.api.controllers;

import java.util.HashMap;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.ISystemMenuTreeService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.SystemMenuTreeValidate;

@RestController
@RequestMapping(value = "/sys/menu")
@MenuCode("SYS02")
public class SystemMenuTreeController extends BaseController {
	@Resource(name = "systemMenuTreeService")
	private ISystemMenuTreeService systemMenuTreeService;

	@RequestMapping(value = "/{menuCode}", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void menuDetail(@PathVariable("menuCode") String menuCode,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		try {
			checkAuthorization(request);

			jsonResult.getData().put("menu",
					systemMenuTreeService.findMenuByCode(menuCode));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ClassNotFoundException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("无法获取菜单信息！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/menu/menulist error", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/menulist", method = RequestMethod.POST)
	public void menuList(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		JsonResult jsonResult = new JsonResult();
		Map<String, Object> queryParam = new HashMap<String, Object>();
		queryParam.put("enable", true);
		queryParam.put(Constants.QUERY_SPEC_CONDITION, "parentMenu is null");
		queryParam
				.put(Constants.QUERY_FILTER + "enableMenuFilter.enable", true);
		queryParam.put(Constants.QUERY_ORDER_BY, "priority");

		try {
			LoginUserInfo loginUser = getLoginUser(request);

			jsonResult.getData().put(
					"menus",
					systemMenuTreeService
							.searchMenuTrees(queryParam, loginUser));
			jsonResult.setSuccess(true);
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/menu/menulist error", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchMenus", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchMenus(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		JsonResult jsonResult = new JsonResult();
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());

		queryParam.put(Constants.QUERY_SPEC_CONDITION, "parentMenu is null");
		queryParam.put(Constants.QUERY_ORDER_BY, "priority");
		queryParam.remove(Constants.QUERY_FIRST_RESULTS);
		queryParam.remove(Constants.QUERY_MAX_RESULTS);

		try {
			checkAuthorization(request);

			jsonResult.getData().putAll(
					systemMenuTreeService.searchMenuTrees(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/menu/searchMenus error", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchParentMenus", method = RequestMethod.POST)
	public void searchParentMenus(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		JsonResult jsonResult = new JsonResult();
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());
		queryParam.put(Constants.QUERY_ORDER_BY, "priority, menuCode");

		try {
			jsonResult.getData().putAll(
					systemMenuTreeService.searchParentMenus(queryParam));
			jsonResult.setSuccess(true);
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/menu/searchParentMenus error", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/searchPerms", method = RequestMethod.POST)
	public void searchPerms(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			jsonResult.getData().put("perms",
					systemMenuTreeService.searchPermissions());
			jsonResult.setSuccess(true);
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/menu/searchPerms error", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/saveMenu", method = RequestMethod.POST)
	@ValidatePermission("INSERT")
	public void saveMenu(HttpServletRequest request,
			HttpServletResponse response,
			@Validated SystemMenuTreeValidate menuTreeValidate,
			BindingResult result) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);

		try {
			checkAuthorization(request);

			validateEnityBean(menuTreeValidate, result);

			systemMenuTreeService.saveMenuTree(menuTreeValidate, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataOccupiedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("菜单编号已存在，不允许重复！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/saveDept error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/updateMenu/{menuCode}", method = RequestMethod.POST)
	@ValidatePermission("UPDATE")
	public void updateMenu(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("menuCode") String menuCode,
			@Validated SystemMenuTreeValidate menuTreeModel,
			BindingResult result) throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try {
			checkAuthorization(request);

			validateEnityBean(menuTreeModel, result);
			systemMenuTreeService.updateMenuTree(menuTreeModel, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/updateMenu error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/deleteMenu/{menuCode}", method = RequestMethod.POST)
	@ValidatePermission("DELETE")
	public void deleteMenu(@PathVariable("menuCode") String menuCode,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			systemMenuTreeService.deleteMenuTree(menuCode);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/deleteMenu error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}
}