package com.bettersoft.api.controllers;

import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.ISystemPermissionService;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.SystemPermissionValidate;

@RestController
@RequestMapping(value = "/sys/perm")
@MenuCode("SYS03")
public class SystemPermissionController extends BaseController {
	@Resource(name = "systemPermissionService")
	private ISystemPermissionService systemPermissionService;

	@RequestMapping(value = "/{permissionCode}")
	@ValidatePermission("SEARCH")
	public void findPerm(@PathVariable("permissionCode") String permissionCode,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			jsonResult.getData().put(
					"perm",
					systemPermissionService
							.findPermissionByCode(permissionCode));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ClassNotFoundException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("无法获取系统权限信息！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/perm/findPerm error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchPerms", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchPerms(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			jsonResult.getData().putAll(
					systemPermissionService.searchPermissions(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dep/searchPerms error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/savePerm", method = RequestMethod.POST)
	@ValidatePermission("INSERT")
	public void savePerm(HttpServletRequest request,
			HttpServletResponse response,
			@Validated SystemPermissionValidate permModel, BindingResult result)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);

		try {
			validateEnityBean(permModel, result);
			systemPermissionService.savePermission(permModel, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataOccupiedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("组织编号已存在，不允许重复！");
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/savePerm error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/updatePerm/{permissionCode}", method = RequestMethod.POST)
	@ValidatePermission("UPDATE")
	public void updatePerm(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("permissionCode") String permissionCode,
			@Validated SystemPermissionValidate permModel, BindingResult result)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try {
			checkAuthorization(request);

			validateEnityBean(permModel, result);
			systemPermissionService.updatePermission(permModel, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/updatePerm error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/deletePerm/{permissionCode}", method = RequestMethod.POST)
	@ValidatePermission("DELETE")
	public void deletePerm(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("permissionCode") String permissionCode)
			throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			systemPermissionService.deletePermission(permissionCode);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("系统错误，请与系统管理员系联！");
			logger.error("sys/dept/deletePerm error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}
}
