package com.bettersoft.api.controllers;

import java.text.MessageFormat;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.ISystemRoleService;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.SystemRoleValidate;

@RestController
@RequestMapping(value = "/sys/role")
@MenuCode("SRC01")
public class SystemRoleController extends BaseController {
	@Resource(name = "systemRoleService")
	private ISystemRoleService systemRoleService;

	@RequestMapping(value = "/{roleCode}")
	@ValidatePermission("SEARCH")
	public void roleDetail(@PathVariable("roleCode") String roleCode,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		try {
			checkAuthorization(request);

			jsonResult.getData().put("role",
					systemRoleService.findRoleByCode(roleCode));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			logger.error("sys/role/roleDetail error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@SuppressWarnings("unchecked")
	@RequestMapping(value = "/searchRoles", method = RequestMethod.POST)
	@ValidatePermission("SEARCH")
	public void searchRoles(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		Map<String, Object> queryParam = RequestMapUtils
				.RequestMapToMap(request.getParameterMap());
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			jsonResult.getData().putAll(
					systemRoleService.searchRoles(queryParam));
			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			logger.error("sys/role/searchRoles error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/searchMenuTrees", method = RequestMethod.POST)
	public void searchMenuTrees(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			jsonResult.getData().put("menus", systemRoleService.searchMenus());

			jsonResult.setSuccess(true);
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			logger.error("sys/role/searchMenuTrees error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/searchDepartments", method = RequestMethod.POST)
	public void searchDepartments(HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			jsonResult.getData().put("depts",
					systemRoleService.searchDepartments());

			jsonResult.setSuccess(true);
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			logger.error("sys/role/searchDepartments error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/saveRole", method = RequestMethod.POST)
	@ValidatePermission("INSERT")
	public void saveRole(HttpServletRequest request,
			HttpServletResponse response,
			@Validated SystemRoleValidate roleModel, BindingResult result)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);

		try {
			checkAuthorization(request);

			validateEnityBean(roleModel, result);
			systemRoleService.saveRole(roleModel, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataOccupiedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(MessageFormat.format("角色编号{0}已存在，不允许重复！",
					roleModel.getRoleCode()));
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			logger.error("sys/role/saveRole error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/updateRole/{roleCode}", method = RequestMethod.POST)
	@ValidatePermission("UPDATE")
	public void updateRole(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("roleCode") String roleCode,
			@Validated SystemRoleValidate roleModel, BindingResult result)
			throws Exception {
		JsonResult jsonResult = new JsonResult();
		LoginUserInfo loginUser = getLoginUser(request);
		try {
			checkAuthorization(request);

			validateEnityBean(roleModel, result);
			systemRoleService.updateRole(roleModel, loginUser);

			jsonResult.setSuccess(true);
		} catch (ValidateAuthorityFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (ValidateDataFailedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(ex.getMessage());
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			logger.error("sys/role/updateDept error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}

	@RequestMapping(value = "/deleteRole/{roleCode}", method = RequestMethod.POST)
	@ValidatePermission("DELETE")
	public void deleteRole(HttpServletRequest request,
			HttpServletResponse response,
			@PathVariable("roleCode") String roleCode) throws Exception {
		JsonResult jsonResult = new JsonResult();

		try {
			checkAuthorization(request);

			systemRoleService.deleteRole(roleCode);

			jsonResult.setSuccess(true);
		} catch (ClassNotFoundException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg("无法获取角色信息！");
		} catch (ValidateDataOccupiedException ex) {
			jsonResult.setSuccess(false);
			jsonResult.setErrMsg(MessageFormat.format("角色编号{0}，已被使用不允许删除！",
					roleCode));
		} catch (Exception ex) {
			jsonResult.setSuccess(false);
			logger.error("sys/role/deleteRole error.", ex);
		} finally {
			renderResponse(jsonResult, response);
		}
	}
}
