package com.bettersoft.api.controllers;

import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.bettersoft.api.annotation.MenuCode;
import com.bettersoft.api.annotation.ValidatePermission;
import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.service.interfaces.ISystemUserService;
import com.bettersoft.api.utils.JsonResult;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.RequestMapUtils;
import com.bettersoft.api.validmodel.LoginAccountValidate;
import com.bettersoft.api.validmodel.SystemUserValidate;

@RestController
@RequestMapping(value = "/sys/user")
@MenuCode("SRC02")
public class SystemUserController extends BaseController {
    @Resource(name = "systemUserService")
    private ISystemUserService systemUserService;

    @RequestMapping(value = "/{userCode}")
    @ValidatePermission("SEARCH")
    public void userDetail(@PathVariable("userCode") String userCode,
                           HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            checkAuthorization(request);

            jsonResult.getData().put("user",
                    systemUserService.findUserByCode(userCode));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(true);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(true);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("sys/user/userDetail error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @SuppressWarnings("unchecked")
    @RequestMapping(value = "/searchUsers", method = RequestMethod.POST)
    @ValidatePermission("SEARCH")
    public void searchUsers(HttpServletRequest request,
                            HttpServletResponse response) throws Exception {
        Map<String, Object> queryParam = RequestMapUtils
                .RequestMapToMap(request.getParameterMap());
        JsonResult jsonResult = new JsonResult();

        try {
            checkAuthorization(request);

            jsonResult.getData().putAll(
                    systemUserService.searchUsers(queryParam));
            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("sys/user/searchUsers error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/searchDepartments", method = RequestMethod.POST)
    public void searchDepartments(HttpServletRequest request,
                                  HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            jsonResult.getData().put("depts",
                    systemUserService.searchDepartments());

            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("sys/user/searchDepartments error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/initParam", method = RequestMethod.POST)
    public void initParam(HttpServletRequest request,
                          HttpServletResponse response) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            jsonResult.setData(systemUserService.initParam());
            jsonResult.setSuccess(true);
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("sys/user/searchRoles error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/saveUser", method = RequestMethod.POST)
    @ValidatePermission("INSERT")
    public void saveUser(HttpServletRequest request,
                         HttpServletResponse response,
                         @Validated SystemUserValidate userModel, BindingResult result)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);

        try {
            checkAuthorization(request);

            validateEnityBean(userModel, result);
            systemUserService.saveUser(userModel, loginUser);

            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataOccupiedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("用户编号已存在，不允许重复！");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("sys/user/saveUser error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/updateUser/{userCode}", method = RequestMethod.POST)
    @ValidatePermission("UPDATE")
    public void updateRole(HttpServletRequest request,
                           HttpServletResponse response,
                           @PathVariable("userCode") String userCode,
                           @Validated SystemUserValidate userModel, BindingResult result)
            throws Exception {
        JsonResult jsonResult = new JsonResult();
        LoginUserInfo loginUser = getLoginUser(request);
        try {
            checkAuthorization(request);

            validateEnityBean(userModel, result);
            systemUserService.updateUser(userModel, loginUser);

            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("sys/user/updateUser error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/deleteUser/{userCode}", method = RequestMethod.POST)
    @ValidatePermission("DELETE")
    public void deleteUser(HttpServletRequest request,
                           HttpServletResponse response,
                           @PathVariable("userCode") String userCode) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            checkAuthorization(request);

            systemUserService.deleteUser(userCode);

            jsonResult.setSuccess(true);
        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("sys/user/deleteUser error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }

    @RequestMapping(value = "/modifyPassword", method = RequestMethod.POST)
    public void modifyPassword(HttpServletRequest request,
                               HttpServletResponse response,
                               @Validated LoginAccountValidate accountValidate,
                               BindingResult result) throws Exception {
        JsonResult jsonResult = new JsonResult();

        try {
            LoginUserInfo loginUser = getLoginUser(request);

            //增加密码校验
            if(!checkPwdValid(accountValidate.getModifyPassword())){
                jsonResult.setErrMsg("新密码不符合校验规则");
                jsonResult.setSuccess(false);
            } else {
                systemUserService.modifyPassword(accountValidate, loginUser);
                jsonResult.setSuccess(true);
            }
//            systemUserService.modifyPassword(accountValidate, loginUser);
//            jsonResult.setSuccess(true);

        } catch (ValidateAuthorityFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg(ex.getMessage());
        } catch (ClassNotFoundException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("无法获取用户信息！");
        } catch (ValidateDataFailedException ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("原密码输入错误！");
        } catch (Exception ex) {
            jsonResult.setSuccess(false);
            jsonResult.setErrMsg("系统错误，请与系统管理员联系！");
            logger.error("sys/user/modifyPassword error.", ex);
        } finally {
            renderResponse(jsonResult, response);
        }
    }
}
