package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.model.LoginUserToken;
import com.bettersoft.api.model.SystemConfigurationBean;
import com.bettersoft.api.utils.*;
import io.jsonwebtoken.Claims;
import org.apache.commons.collections.map.HashedMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.web.socket.TextMessage;

import com.bettersoft.api.dao.interfaces.IAuthorizationDao;
import com.bettersoft.api.model.SystemUserBean;
import com.bettersoft.api.service.interfaces.IAuthorizationService;
import com.bettersoft.api.validmodel.LoginAccountValidate;
import com.bettersoft.api.websocket.BettersoftWebSocketHandler;

@Service("authorizationService")
public class AuthorizationService extends BaseService implements
        IAuthorizationService {
    @Resource(name = "authorizationDao")
    private IAuthorizationDao authorizationDao;
    @Autowired(required = false)
    private BettersoftWebSocketHandler bettersoftHandler;
    @Resource(name = "redisCacheUtil")
    private RedisCacheUtil redisCacheUtil;

    @Autowired
    private SystemConfigurationService service;

    @SuppressWarnings({"rawtypes", "unchecked"})
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void loginUser(LoginUserInfo loginUser, boolean force)
            throws Exception {
        if (force && null != bettersoftHandler) {
            Map loginMap = redisCacheUtil
                    .getCacheMap(Constants.LOGIN_SESSION_USER
                            + loginUser.getUserCode());
            for (Iterator<String> itKey = loginMap.keySet().iterator(); itKey
                    .hasNext(); ) {
                String sessionId = itKey.next();
                bettersoftHandler.pushMessageToAccount(
                        Constants.WEBSOCKET_SESSION_KEY + sessionId,
                        new TextMessage("LOGOUT_FORCE"));
                redisCacheUtil.removeCacheByKey(Constants.LOGIN_TOKEN_KEY + loginUser.getToken());
                redisCacheUtil.removeCacheByKey(Constants.LOGIN_SESSION_USER
                        + loginUser.getUserCode(), sessionId);
            }
        }

        LoginUserToken userToken = new LoginUserToken();
        userToken.setLoginId(StringDigestUtil.generalUUID());
        userToken.setLoginUser(loginUser.getUserCode());
        userToken.setLoginTicket(loginUser.getTicket());
        userToken.setToken(loginUser.getToken());
        userToken.setRefreshToken(loginUser.getRefreshToken());
        userToken.setExpireTime(loginUser.getExpireTime());
        userToken.setSessionId(loginUser.getSessionId());
        userToken.setPlatform(loginUser.getPlatform());

        authorizationDao.save(userToken);

        redisCacheUtil.setCache(Constants.LOGIN_TOKEN_KEY + userToken.getToken(), loginUser, Constants.SESSION_EXPIRE_TIME);
        redisCacheUtil.setCache(
                Constants.LOGIN_SESSION_USER + loginUser.getUserCode(),
                loginUser.getSessionId(), loginUser,
                Constants.SESSION_EXPIRE_TIME);
        authorizationDao.save(generalLogOperator(Constants.LOGIN_OPER_TYPE,
                loginUser));
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void logoutUser(LoginUserInfo loginUser) throws Exception {
        redisCacheUtil.removeCacheByKey(Constants.LOGIN_TOKEN_KEY + loginUser.getToken());
        redisCacheUtil.removeCacheByKey(Constants.LOGIN_SESSION_USER
                + loginUser.getUserCode(), loginUser.getSessionId());
        authorizationDao.save(generalLogOperator(Constants.LOGOUT_OPER_TYPE,
                loginUser));
    }

    public Claims validateLoginUser(String token) throws Exception {
        Claims claims = JwtUtils.verifyToken(token, redisCacheUtil);

        if (null != claims)
            return claims;

        LoginUserToken userToken = authorizationDao.findLoginToken(token);
        if (null == userToken)
            throw new ClassNotFoundException();

        claims = JwtUtils.parseToken(userToken.getLoginTicket());

        LoginUserInfo loginUser = EntityUtil.convertMapToEntity(LoginUserInfo.class, claims);

        redisCacheUtil.setCache(Constants.LOGIN_TOKEN_KEY + userToken.getToken(), loginUser, Constants.SESSION_EXPIRE_TIME);
        redisCacheUtil.setCache(
                Constants.LOGIN_SESSION_USER + loginUser.getUserCode(),
                loginUser.getSessionId(), loginUser,
                Constants.SESSION_EXPIRE_TIME);

        return claims;
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public LoginUserInfo refreshLoginUser(String refreshToken) throws Exception {
        LoginUserToken userToken = authorizationDao.findLoginTokenByRefresh(refreshToken);
        if (null == userToken)
            throw new ValidateAuthorityFailedException();

        SystemUserBean chkUser = authorizationDao.findEntity(SystemUserBean.class, userToken.getLoginUser());
        if (null == chkUser)
            throw new ClassNotFoundException();

        Claims claims = JwtUtils.parseToken(userToken.getLoginTicket());
        LoginUserInfo loginUser = EntityUtil.convertMapToEntity(LoginUserInfo.class, claims);
        loginUser.setToken(StringDigestUtil.generalUUID());
        loginUser.setRefreshToken(StringDigestUtil.generalUUID());
        loginUser.setExpireTime(new Timestamp(System.currentTimeMillis() + Constants.LOGIN_EXPIRE_TIME));
        loginUser.setTicket(JwtUtils.createToken(loginUser, Constants.LOGIN_EXPIRE_TIME));

        userToken.setLoginTicket(loginUser.getTicket());
        userToken.setToken(loginUser.getToken());
        userToken.setRefreshToken(loginUser.getRefreshToken());
        userToken.setExpireTime(loginUser.getExpireTime());
        authorizationDao.update(userToken);

        redisCacheUtil.removeCacheByKey(Constants.LOGIN_SESSION_USER
                + loginUser.getUserCode(), loginUser.getSessionId());
        redisCacheUtil.removeCacheByKey(Constants.LOGIN_TOKEN_KEY);
        redisCacheUtil.setCache(Constants.LOGIN_TOKEN_KEY + userToken.getToken(), loginUser, Constants.SESSION_EXPIRE_TIME);
        redisCacheUtil.setCache(
                Constants.LOGIN_SESSION_USER + loginUser.getUserCode(),
                loginUser.getSessionId(), loginUser,
                Constants.SESSION_EXPIRE_TIME);
        return loginUser;
    }

    // @Transactional(propagation = Propagation.REQUIRED, isolation =
    // Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    // public void register(MembershipValidate registerValidate) throws
    // Exception {
    // MembershipBean membership = authorizationDao
    // .findMembershipByMobile(registerValidate.getMobile());
    //
    // if (null != membership)
    // throw new ValidateDataOccupiedException();
    //
    // SystemConfigurationBean advLimitCnf = authorizationDao
    // .findConfigurationByCode(Constants.CONF_ADV_LIMIT);
    //
    // int advLimit = 0;
    // if (null != advLimitCnf
    // && !StringUtils.isEmpty(advLimitCnf.getConfigValue())) {
    // try {
    // advLimit = Integer.valueOf(advLimitCnf.getConfigValue());
    // } catch (Exception ex) {
    // advLimit = 0;
    // }
    // }
    //
    // membership = new MembershipBean();
    //
    // EntityUtil.copyEntityProperty(registerValidate, membership);
    //
    // membership.setMemberId(generalMemberId());
    //
    // MembershipLevelBean defaultLevel = authorizationDao.findDefaultLevel();
    // membership.setPassword(registerValidate.getPassword());
    // membership.setStatus("40");
    //
    // if (null != defaultLevel) {
    // membership.setLevelCode(defaultLevel.getLevelCode());
    // membership.setMembershipLevel(defaultLevel);
    // }
    //
    // if (StringUtils.isEmpty(membership.getGender()))
    // membership.setGender("F");
    //
    // membership.setRegistDateTime(new Timestamp(System.currentTimeMillis()));
    // if (advLimit != 0) {
    // Calendar cal = Calendar.getInstance();
    // cal.setTime(new Date());
    // cal.add(Calendar.DATE, advLimit);
    //
    // membership.setAdvLimitDate(cal.getTime());
    // }
    //
    // membership.setCreateBy(registerValidate.getMobile());
    // membership.setCreateDateTime(new Timestamp(System.currentTimeMillis()));
    // membership.setCreateUserName(StringUtils.isEmpty(registerValidate
    // .getNickName()) ? registerValidate.getMobile()
    // : registerValidate.getNickName());
    //
    // authorizationDao.save(membership);
    // authorizationDao.save(generalLogOperator(Constants.REGISTER_OPER_TYPE,
    // new LoginUserInfo(), membership));
    // }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void forgetPassword(LoginAccountValidate accountValidate)
            throws Exception {
        SystemUserBean user = authorizationDao.findEntity(SystemUserBean.class,
                accountValidate.getAccount());
        if (null == user)
            throw new ClassNotFoundException();

        user.setPassword(accountValidate.getModifyPassword());

        authorizationDao.update(user);
    }

    @Override
    @Transactional
    public boolean checkIsLock(LoginAccountValidate loginAccount) throws Exception {
        SystemUserBean chkUser = authorizationDao.findEntity(
                SystemUserBean.class, loginAccount.getAccount());

        Integer strLockCount = 0;

        SystemConfigurationBean lockCount = redisCacheUtil.getCache(Constants.SYSCONFIG_KEY,"SYS_INIT_LOCK_COUNT");

        if(ObjectUtils.isEmpty(lockCount)){
            lockCount = service.findConfigurationByCodeNoException("SYS_INIT_LOCK_COUNT");
        }

        if(ObjectUtils.isEmpty(lockCount)){
            strLockCount = 5;
        } else {
            strLockCount = Integer.parseInt(lockCount.getConfigValue());
        }

        if (!ObjectUtils.isEmpty(chkUser) && chkUser.getPwdErrorCount() >= strLockCount) {
            Timestamp unlockTime = chkUser.getFreeLockTime();
            Timestamp now = new Timestamp(System.currentTimeMillis());
            if (now.after(unlockTime)) {
                chkUser.setPwdErrorCount(0);
                authorizationDao.saveOrUpdate(chkUser);
                return false;
            }
        } else if (!ObjectUtils.isEmpty(chkUser) && chkUser.getPwdErrorCount() < strLockCount) {
            return false;
        }
        return !ObjectUtils.isEmpty(chkUser);
    }

    @Override
    @Transactional
    public void loginError(LoginAccountValidate loginAccount) throws Exception {
        SystemUserBean chkUser = authorizationDao.findEntity(
                SystemUserBean.class, loginAccount.getAccount());
        Integer strLockTime = 0;
        Integer strLockCount = 0;

        SystemConfigurationBean lockTime = redisCacheUtil.getCache(Constants.SYSCONFIG_KEY,"SYS_INIT_LOCK_TIME");
        SystemConfigurationBean lockCount = redisCacheUtil.getCache(Constants.SYSCONFIG_KEY,"SYS_INIT_LOCK_COUNT");

        if(ObjectUtils.isEmpty(lockTime)){
            lockTime = service.findConfigurationByCodeNoException("SYS_INIT_LOCK_TIME");
        }
        if(ObjectUtils.isEmpty(lockCount)){
            lockCount = service.findConfigurationByCodeNoException("SYS_INIT_LOCK_COUNT");
        }

        if(ObjectUtils.isEmpty(lockTime)){
            strLockTime = 1800000;
        } else {
            strLockTime = Integer.parseInt(lockTime.getConfigValue()) * 60 * 1000;
        }
        if(ObjectUtils.isEmpty(lockCount)){
            strLockCount = 5;
        } else {
            strLockCount = Integer.parseInt(lockCount.getConfigValue());
        }

        if(ObjectUtils.isEmpty(chkUser)){
            return;
        }
        chkUser.setPwdErrorCount(chkUser.getPwdErrorCount() + 1);
        if (chkUser.getPwdErrorCount() >= strLockCount) {
            chkUser.setFreeLockTime(new Timestamp(System.currentTimeMillis() + strLockTime));
        }
        authorizationDao.saveOrUpdate(chkUser);
    }

    @Override
    @Transactional
    public void resetIsLock(LoginAccountValidate loginAccount) {
        SystemUserBean chkUser = authorizationDao.findEntity(
                SystemUserBean.class, loginAccount.getAccount());
        chkUser.setPwdErrorCount(0);
        authorizationDao.saveOrUpdate(chkUser);
    }

    public boolean checkEntityOccupied(Serializable id) throws Exception {
        return false;
    }
}
