package com.bettersoft.api.service.implement;

import java.io.Serializable;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.mail.Store;

import com.bettersoft.api.model.*;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.bettersoft.api.dao.interfaces.ISystemUserDao;
import com.bettersoft.api.exceptions.ValidateDataFailedException;
import com.bettersoft.api.exceptions.ValidateDataOccupiedException;
import com.bettersoft.api.exceptions.ValidateFailedException;
import com.bettersoft.api.service.interfaces.ISystemUserService;
import com.bettersoft.api.utils.Constants;
import com.bettersoft.api.utils.EntityUtil;
import com.bettersoft.api.utils.LoginUserInfo;
import com.bettersoft.api.utils.PageInfoUtils;
import com.bettersoft.api.utils.StringDigestUtil;
import com.bettersoft.api.validmodel.LoginAccountValidate;
import com.bettersoft.api.validmodel.SystemUserValidate;

@Service("systemUserService")
public class SystemUserService extends BaseService implements
        ISystemUserService {
    @Resource(name = "systemUserDao")
    private ISystemUserDao systemUserDao;

    public SystemUserValidate findUserByCode(String userCode) throws Exception {
        SystemUserBean user = systemUserDao.findEntity(SystemUserBean.class,
                userCode);
        if (null == user)
            throw new ClassNotFoundException();

        SystemUserValidate userValidate = (SystemUserValidate) EntityUtil
                .cloneToEntity(SystemUserValidate.class, user);

        if (null != user.getAuthRoles() && !user.getAuthRoles().isEmpty()) {
            List<String> roles = new ArrayList<String>();
            for (SystemUserRoleBean userRole : user.getAuthRoles()) {
                if (null != userRole.getSystemRole())
                    roles.add(userRole.getRoleCode());
            }

            userValidate.setRoles(StringUtils.join(roles, ","));
        }

        if (null != user.getStores() && !user.getStores().isEmpty()) {
            List<String> stores = new ArrayList<String>();
            for (SystemUserStoreBean userStore: user.getStores()) {
                if (null != userStore.getStore())
                    stores.add(userStore.getStoreCode());
            }

            userValidate.setStores(StringUtils.join(stores, ","));
        }

        return userValidate;
    }

    public Map<String, Object> searchUsers(Map<String, Object> queryParam)
            throws Exception {
        queryParam.put("supper", false);
        List<SystemUserBean> users = systemUserDao.searchUsers(queryParam);

        if (null != users && !users.isEmpty()) {
            for (SystemUserBean user : users) {
                if (null != user.getDepartment()) {
                    user.setDepartmentName(user.getDepartment()
                            .getDepartmentDesc());
                }
            }
        }
        Number count = systemUserDao.searchUserCount(queryParam);

        return new PageInfoUtils(queryParam, users, count).getPageInfoMap();
    }

    public Map<String, Object> initParam() throws Exception {
        Map<String, Object> resultMap = new HashMap<String, Object>();
        resultMap.put("roles", systemUserDao.searchRoles());
        resultMap.put("stores", systemUserDao.searchStores());

        return resultMap;
    }

    public List<SystemDepartmentBean> searchDepartments() throws Exception {
        return systemUserDao.searchDepartments();
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void saveUser(SystemUserValidate userModel, LoginUserInfo loginUser)
            throws Exception {
        SystemUserBean user = (SystemUserBean) EntityUtil.cloneToEntity(
                SystemUserBean.class, userModel);
        user.setUserCode(userModel.getUserCode());

        if (checkEntityOccupied(user.getUserCode()))
            throw new ValidateDataOccupiedException();

        SystemConfigurationBean initPwdCfg = systemUserDao
                .findConfigurationByCode("SYS_INIT_PWD");

        //新增用户时增加对初始密码的校验
        if (null == initPwdCfg)
            throw new ValidateFailedException();
        if (!checkPwdValid(initPwdCfg.getConfigValue()))
            throw new ValidateDataFailedException("初始密码不符合校验规则，新增用户失败，请到[系统配置页面]修改初始密码");
//        if (null == initPwdCfg)
//            throw new ValidateFailedException);

        user.setPassword(StringDigestUtil.encodeMD5Hex(initPwdCfg
                .getConfigValue()));

        if (!StringUtils.isEmpty(user.getDepartmentCode())) {
            SystemDepartmentBean department = systemUserDao.findEntity(
                    SystemDepartmentBean.class, user.getDepartmentCode());

            if (null == department)
                throw new ClassNotFoundException();
        }

        if (!StringUtils.isEmpty(userModel.getRoles())) {
            String[] roles = StringUtils.split(userModel.getRoles(), ",");
            for (String roleCode : roles) {
                SystemRoleBean role = systemUserDao.findEntity(
                        SystemRoleBean.class, roleCode);

                if (null == role)
                    throw new ClassNotFoundException();

                SystemUserRoleBean userRole = new SystemUserRoleBean();

                userRole.setUserCode(user.getUserCode());
                userRole.setRoleCode(roleCode);
                userRole.setCreateDateTime(new Timestamp(System
                        .currentTimeMillis()));

                user.getAuthRoles().add(userRole);
            }
        }

        if (!StringUtils.isEmpty(userModel.getStores())) {
            String[] stores = StringUtils.split(userModel.getStores(), ",");
            for (String storeCode : stores) {
                StoreBean store = systemUserDao.findEntity(StoreBean.class, storeCode);
                if (null == store)
                    throw new ClassNotFoundException();

                SystemUserStoreBean userStore = new SystemUserStoreBean();

                userStore.setUserCode(user.getUserCode());
                userStore.setStoreCode(storeCode);
                userStore.setCreateDateTime(new Timestamp(System.currentTimeMillis()));

                user.getStores().add(userStore);
            }
        }

        user.setCreateBy(loginUser.getUserCode());
        user.setCreateDateTime(new Timestamp(System.currentTimeMillis()));
        user.setCreateUserName(loginUser.getUserName());

        systemUserDao.save(user);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void updateUser(SystemUserValidate userModel, LoginUserInfo loginUser)
            throws Exception {
        String userCode = userModel.getUserCode();

        SystemUserBean editUser = systemUserDao.findEntity(
                SystemUserBean.class, userCode);

        EntityUtil.compareCopyEntity(userModel, editUser);

        editUser.getAuthRoles().clear();
        if (!StringUtils.isEmpty(userModel.getRoles())) {
            String[] roles = StringUtils.split(userModel.getRoles(), ",");
            Map<String, Object> queryParam = new HashMap<String, Object>();
            queryParam.put("userCode", editUser.getUserCode());
            for (String roleCode : roles) {
                queryParam.put("roleCode", roleCode);
                SystemUserRoleBean userRole = systemUserDao
                        .findUserRole(queryParam);

                if (null == userRole) {
                    userRole = new SystemUserRoleBean();

                    SystemRoleBean role = systemUserDao.findEntity(
                            SystemRoleBean.class, roleCode);

                    if (null == role)
                        throw new ClassNotFoundException();

                    userRole.setUserCode(editUser.getUserCode());
                    userRole.setRoleCode(roleCode);
                    userRole.setCreateDateTime(new Timestamp(System
                            .currentTimeMillis()));
                }

                editUser.getAuthRoles().add(userRole);
            }
        }

        editUser.getStores().clear();
        if (!StringUtils.isEmpty(userModel.getStores())) {
            String[] stores = StringUtils.split(userModel.getStores(), ",");
            Map<String, Object> queryParam = new HashMap<String, Object>();
            queryParam.put("userCode", editUser.getUserCode());
            for (String storeCode : stores) {
                queryParam.put("storeCode", storeCode);
                SystemUserStoreBean userStore = systemUserDao.findUserStore(queryParam);

                if (null == userStore) {
                    userStore = new SystemUserStoreBean();

                    StoreBean store = systemUserDao.findEntity(StoreBean.class, storeCode);
                    if (null == store)
                        throw new ClassNotFoundException();

                    userStore.setUserCode(editUser.getUserCode());
                    userStore.setStoreCode(storeCode);
                    userStore.setCreateDateTime(new Timestamp(System.currentTimeMillis()));

                    editUser.getStores().add(userStore);
                }
            }
        }

        editUser.setLastModifyBy(loginUser.getUserCode());
        editUser.setLastModifyDateTime(new Timestamp(System.currentTimeMillis()));
        editUser.setLastModifyUserName(loginUser.getUserName());

        systemUserDao.update(editUser);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void deleteUser(String userCode) throws Exception {
        SystemUserBean delUser = systemUserDao.findEntity(SystemUserBean.class,
                userCode);
        if (null == delUser)
            throw new ClassNotFoundException();

        systemUserDao.delete(delUser);
    }

    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT, readOnly = false, rollbackFor = Exception.class)
    public void modifyPassword(LoginAccountValidate accountValidate,
                               LoginUserInfo loginUser) throws Exception {
        SystemUserBean user = systemUserDao.findEntity(SystemUserBean.class,
                accountValidate.getAccount());

        if (null == user)
            throw new ClassNotFoundException();

        if (!user.getPassword().equals(
                StringDigestUtil.encodeMD5Hex(accountValidate.getPassword())))
            throw new ValidateDataFailedException();

        user.setPassword(StringDigestUtil.encodeMD5Hex(accountValidate
                .getModifyPassword()));

        systemUserDao.update(user);
        systemUserDao.save(generalLogOperator(Constants.MODPWD_OPER_TYPE,
                loginUser));
    }

    public boolean checkEntityOccupied(Serializable id) throws Exception {
        SystemUserBean existsUser = systemUserDao.findEntity(
                SystemUserBean.class, id);

        return null != existsUser;
    }

}
