package com.bettersoft.api.utils;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

@SuppressWarnings({ "unchecked", "rawtypes", "unused" })
public class CacheUtil {

	private final static Logger log = LoggerFactory.getLogger(CacheUtil.class);

	// 默认为最大的时间(6个月)
	private static final int EXPIRETIME_DEFAULT = 2592000;

	private static Map<String, CacheData> CACHE_DATA = new ConcurrentHashMap<>();

	public static <T> T get(String key, Load<T> load, int expire) {
		T data = get(key);
		if (data == null && load != null) {
			data = load.load();
			if (data != null) {
				put(key, data, expire);
			}
		}
		return data;
	}

	public static <T> T get(String key) {
		CacheData<T> data = CACHE_DATA.get(key);
		if (data != null
				&& (data.getExpire() <= 0 || data.getSaveTime() >= new Date()
						.getTime())) {
			return data.getData();
		}
		return null;
	}

	public static <T> CacheData put(String key, T data) {
		return CACHE_DATA.put(key, new CacheData(data, EXPIRETIME_DEFAULT));
	}

	public static <T> CacheData put(String key, T data, int expire) {
		return CACHE_DATA.put(key, new CacheData(data, expire));
	}

	public static CacheData remove(String key) {
		return CACHE_DATA.remove(key);
	}

	public static void clearAll() {
		CACHE_DATA.clear();
	}

	public interface Load<T> {
		T load();
	}

	private static class CacheData<T> {
		CacheData(T t, int expire) {
			this.data = t;
			this.expire = expire <= 0 ? 0 : expire * 1000;
			this.saveTime = new Date().getTime() + this.expire;
		}

		private T data;
		private long saveTime;
		private long expire;

		public T getData() {
			return data;
		}

		public long getExpire() {
			return expire;
		}

		public long getSaveTime() {
			return saveTime;
		}
	}
}
