package com.bettersoft.api.utils;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.bettersoft.api.annotation.Cascade;
import com.bettersoft.api.annotation.CloneType;
import com.bettersoft.api.annotation.FieldCommon;
import com.bettersoft.api.annotation.Ignore;
import com.bettersoft.api.annotation.NotClone;
import com.bettersoft.api.annotation.NotCompare;
import com.bettersoft.api.annotation.NotCopy;
import com.bettersoft.api.annotation.NotMapped;

public class EntityUtil {
    private static Log logger = LogFactory.getLog(EntityUtil.class);

    public static <T> List<Map<String, Object>> convertEntityListToMapList(
            List<T> entities, String... properties) {
        List<Map<String, Object>> mapList = new ArrayList<Map<String, Object>>();
        if (null != entities && !entities.isEmpty()) {
            for (T entity : entities)
                mapList.add(convertEntityToMap(entity, properties));
        }
        return mapList;
    }

    @SuppressWarnings("rawtypes")
    public static <T> List<Map<String, Object>> convertEntityListToMapList(
            String className, Collection<T> entities)
            throws ClassNotFoundException {
        Class clazz = Class.forName(className);

        return convertEntityListToMapList(clazz, entities);
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    public static <T> List<Map<String, Object>> convertEntityListToMapList(
            Class clazz, Collection<T> entities) {
        List<Map<String, Object>> mapList = new ArrayList<Map<String, Object>>();

        try {
            Field[] fields = getDeclareFields(clazz);
            if (null != entities && !entities.isEmpty()) {
                for (T entity : entities) {
                    Map<String, Object> map = new HashMap<String, Object>();
                    if (null != fields && 0 < fields.length) {
                        for (Field field : fields) {
                            if ((field.getModifiers() == Modifier.PRIVATE || field
                                    .getModifiers() == Modifier.PROTECTED)
                                    && field.getModifiers() != Modifier.TRANSIENT) {
                                if (field.getType() == Set.class
                                        || field.getType() == List.class) {
                                    Collection c = (Collection) getFieldValue(
                                            entity, field);
                                    ParameterizedType pt = (ParameterizedType) field
                                            .getGenericType();
                                    Class innerClazz = (Class) pt
                                            .getActualTypeArguments()[0];
                                    map.put(field.getName(),
                                            convertEntityListToMapList(
                                                    innerClazz, c));
                                } else {
                                    map.put(field.getName(),
                                            getFieldValue(entity, field));
                                }
                            }
                        }
                    }
                    mapList.add(map);
                }
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }

        return mapList;
    }

    public static <T> List<Map<String, Object>> convertEntityListToMapList(Collection<T> entities) {
        List<Map<String, Object>> entityMaps = new ArrayList<Map<String, Object>>();
        for (T entity : entities) {
            Map<String, Object> entityMap = convertEntityToMap(entity);

            entityMaps.add(entityMap);
        }

        return entityMaps;
    }

    @SuppressWarnings("unchecked")
    public static <T> Map<Object, T> convertEntityList2MapKey(List<T> entities,
                                                              String keyProperty) {
        Map<Object, T> resultMap = new HashMap<Object, T>();
        if (null != entities && !entities.isEmpty()) {
            for (T entity : entities) {
                T key = (T) getPropertyValue(entity, keyProperty);
                resultMap.put(key, entity);
            }
        }

        return resultMap;
    }

    public static <T> Map<String, Object> convertEntityToMap(T entity,
                                                             String... properties) {
        Map<String, Object> map = new HashMap<String, Object>();
        for (String property : properties)
            map.put(property, getPropertyValue(entity, property));
        return map;
    }

    public static <T> Map<String, Object> convertEntityToMap(T entity) {
        if (null == entity)
            return null;

        Map<String, Object> map = new HashMap<String, Object>();
        Field[] fields = getDeclareFields(entity.getClass());
        if (null != fields && 0 < fields.length) {
            for (Field field : fields) {
                if (isTransient(field))
                    continue;

                if (isNotMapped(field))
                    continue;

                if (hasCascade(field)) {
                    try {
                        map.put(field.getName(),
                                getFieldValueByCascade(entity, field));
                    } catch (Exception ex) {
                        logger.error("EntityUril.getFieldValueByCascade error",
                                ex);
                        map.put(field.getName(), null);

                    }
                } else
                    map.put(field.getName(),
                            getPropertyValue(entity, field.getName()));
            }
        }
        return map;
    }

    @SuppressWarnings({"unchecked", "rawtypes"})
    public static <T> T convertMapToEntity(Class<T> clazz,
                                           Map<String, Object> map) {
        T resultObj = null;
        try {
            resultObj = clazz.newInstance();

            for (Iterator<String> itKey = map.keySet().iterator(); itKey
                    .hasNext(); ) {
                String key = itKey.next();
                if (StringUtils.isEmpty(key))
                    continue;
                if (isIgnore(getDeclaredField(resultObj, key)))
                    continue;

                Object propertyValue = MapUtils.getObject(map, key);
                if (propertyValue instanceof List) {
                    List<Map<String, Object>> collectionMaps = (List<Map<String, Object>>) propertyValue;
                    Field field = getDeclaredField(resultObj, key);
                    if (null != field) {
                        ParameterizedType pType = (ParameterizedType) field
                                .getGenericType();
                        Collection<T> collectionValue = (Collection<T>) getFieldValue(
                                resultObj, key);
                        collectionValue.clear();
                        for (Object el : collectionMaps) {
                            T innerEntity = null;
                            if (el instanceof Map) {
                                innerEntity = (T) convertMapToEntity(
                                        (Class) pType.getActualTypeArguments()[0],
                                        (Map) el);
                            } else {
                                innerEntity = (T) el;
                            }

                            setParentEntity(resultObj, innerEntity);
                            collectionValue.add(innerEntity);
                        }
                    }

                    continue;
                }
                setFieldValue(resultObj, key, propertyValue);
            }
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return resultObj;
    }

    public static <T> T convertMapToEntityInMerge(Class<T> clazz,
                                                  Map<String, Object> map, T sourceEntity) {
        T targetEntity = convertMapToEntity(clazz, map);
        Field[] fields = getDeclareFields(targetEntity.getClass());
        try {
            for (Field field : fields) {
                if (isIgnore(field))
                    continue;

                Object sourceValue = getFieldValue(sourceEntity, field);
                Object targetValue = getFieldValue(targetEntity, field);

                if (null == targetValue && !map.containsKey(field.getName()))
                    setFieldValue(targetEntity, field, sourceValue);
            }

        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return targetEntity;
    }

    public static List<Map<String, Object>> convertList2MapList(
            List<Object[]> list, String... fields) {
        List<Map<String, Object>> result = new ArrayList<Map<String, Object>>();
        for (Object[] objs : list) {
            result.add(convertArray2Map(objs, fields));
        }

        return result;
    }

    public static Map<String, Object> convertArray2Map(Object[] objs,
                                                       String... fields) {
        Map<String, Object> resultMap = new HashMap<String, Object>();
        for (int i = 0; i < fields.length; i++) {
            resultMap.put(fields[i], objs[i]);
        }

        return resultMap;
    }

    public static void compareCopyEntity(Object srObj, Object tgObj) {
        Field[] fields = getDeclareFields(srObj.getClass());
        try {
            for (Field field : fields) {
                if (isNotCopy(field))
                    continue;

                Object sourceValue = getFieldValue(srObj, field.getName());
                Object targetValue = getFieldValue(tgObj, field.getName());
                if (null == sourceValue && null != targetValue)
                    setFieldValue(tgObj, field.getName(), sourceValue);
                else if (null != sourceValue && null == targetValue)
                    setFieldValue(tgObj, field.getName(), sourceValue);
                else if (null != sourceValue && null != targetValue
                        && !sourceValue.equals(targetValue))
                    setFieldValue(tgObj, field.getName(), sourceValue);
            }
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        }
    }

    public static <T> T copyEntityProperty(T sourceEntity, T targetEntity,
                                           String... ignoreFields) {
        try {
            Field[] fields = getDeclareFields(targetEntity.getClass());
            for (Field field : fields) {
                if (isNotCopy(field))
                    continue;
                if (isIgnore(field, ignoreFields))
                    continue;

                setFieldValue(targetEntity, field,
                        getFieldValue(sourceEntity, field.getName()));
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        return targetEntity;
    }

    @SuppressWarnings({"unchecked", "rawtypes"})
    public static <T> T cloneEntity(T entity, CloneType... cloneTypes) {
        T clone = null;
        try {
            if (null != cloneTypes && 0 < cloneTypes.length
                    && cloneTypes[0] == CloneType.MODIFY)
                clone = (T) Class.forName(
                        entity.getClass().getName() + "Modify").newInstance();
            else
                clone = (T) entity.getClass().newInstance();

            Field[] fields = getDeclareFields(clone.getClass());
            for (Field field : fields) {
                if (isNotClone(field))
                    continue;

                if (field.getType() == Set.class
                        || field.getType() == List.class) {
                    Collection collection = (Collection) getFieldValue(entity,
                            field.getName());
                    Collection cloneCollection = (Collection) getFieldValue(
                            clone, field.getName());
                    for (Object obj : collection) {
                        Object cloneObj = cloneEntity(obj, cloneTypes);

                        if (null != cloneObj) {
                            setParentEntity(clone, cloneObj);
                            cloneCollection.add(cloneObj);
                        }
                    }

                    setFieldValue(clone, field, cloneCollection);
                } else if (hasCascade(field)) {
                    setFieldValue(clone, field,
                            getFieldValueByCascade(entity, field));
                } else {
                    setFieldValue(clone, field,
                            getFieldValue(entity, field.getName()));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("EntityUtil cloneEntity error.", e);
        }

        return clone;
    }

    public static <T> Object cloneToEntity(Class<?> tarClazz, T srcEntity) {
        Object clone = null;

        try {
            clone = tarClazz.newInstance();
            Field[] fields = getDeclareFields(clone.getClass());
            for (Field field : fields) {
                if (isNotClone(field))
                    continue;

                Field srcField = getDeclaredField(srcEntity, field.getName());
                if (null != srcField && field.getType() == srcField.getType()) {
                    Object value = null;
                    if (hasCascade(srcField))
                        value = getFieldValueByCascade(srcEntity, srcField);
                    else
                        value = getFieldValue(srcEntity, srcField);

                    setFieldValue(clone, field, value);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("EntityUtil cloneEntity error.", e);
        }

        return clone;
    }

    public static <T> boolean equalsEntity(T entity1, T entity2) {
        Field[] fields = getDeclareFields(entity1.getClass());
        boolean result = true;
        for (Field field : fields) {
            if (isNotCompare(field))
                continue;
            Object value1 = getPropertyValue(entity1, field.getName());
            Object value2 = getPropertyValue(entity2, field.getName());

            if (null != value1 && null != value2) {
                if (value1 instanceof Timestamp || value1 instanceof Date)
                    result = ((Date) value1).getTime() == ((Date) value2)
                            .getTime();
                else if (value1 instanceof BigDecimal)
                    result = ((BigDecimal) value1).floatValue() == ((BigDecimal) value2)
                            .floatValue();
                else
                    result = value1.equals(value2);
            } else if (null == value1 && null != value2)
                result = false;
            else if (null != value1 && null == value2)
                result = false;

            if (!result)
                break;
        }

        return result;
    }

    public static <T> String getFieldCommon(T entity, String fieldName)
            throws Exception {
        Field field = getDeclaredField(entity, fieldName);

        if (null != field) {
            FieldCommon fieldCommon = field.getAnnotation(FieldCommon.class);

            return null != fieldCommon ? fieldCommon.common() : fieldName;
        }

        return fieldName;
    }

    public static <T> Object getPropertyValue(T entity, String property) {
        Object value = null;
        try {
            Field field = getDeclaredField(entity, property);

            if (null != field) {
                field.setAccessible(true);
                value = field.get(entity);
                if (null == value)
                    value = getFieldValueByMethod(entity, property);
            } else {
                value = getFieldValueByMethod(entity, property);
            }
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        return value;
    }

    @SuppressWarnings("unchecked")
    public static <T> Field[] getDeclareFields(Class<T> clazz) {
        List<Field> fields = new ArrayList<Field>();

        for (; clazz != Object.class; clazz = (Class<T>) clazz.getSuperclass()) {
            try {
                Field[] fieldArray = clazz.getDeclaredFields();
                for (Field field : fieldArray) {
                    if (field.getModifiers() == Modifier.PRIVATE
                            || field.getModifiers() == Modifier.PRIVATE
                            + Modifier.TRANSIENT
                            || field.getModifiers() == Modifier.PROTECTED) {
                        fields.add(field);
                    }
                }
            } catch (Exception e) {
            }
        }

        return fields.toArray(new Field[0]);
    }

    public static <T> Method getDeclareMethod(Class<T> clazz, String methodName) {
        Method[] methods = clazz.getDeclaredMethods();
        if (null != methods && 0 < methods.length) {
            for (Method method : methods) {
                if (method.getName().equals(methodName))
                    return method;
            }
        }

        return null;
    }

    private static String getGetterMethodName(String property) {
        if (property.length() <= 1)
            return "get" + property.toUpperCase();

        return "get" + property.substring(0, 1).toUpperCase()
                + property.substring(1, property.length());
    }

    /*
     * private static String getSetterMethodName(String property) { if (1 >=
     * property.length()) return "set" + property.toUpperCase();
     *
     * return "set" + property.substring(0, 1).toUpperCase() +
     * property.substring(1, property.length()); }
     */
    private static Field getDeclaredField(Object object, String fieldName) {
        Field field = null;

        Class<?> clazz = object.getClass();

        for (; clazz != Object.class; clazz = clazz.getSuperclass()) {
            try {
                field = clazz.getDeclaredField(fieldName);
                return field;
            } catch (Exception e) {
            }
        }

        return null;
    }

    @SuppressWarnings("unchecked")
    private static <T> T getFieldValue(T entity, Field field)
            throws IllegalArgumentException, IllegalAccessException {
        field.setAccessible(true);
        Object value = field.get(entity);
        if (null == value)
            value = getPropertyValue(entity, field.getName());

        return (T) value;
    }

    private static <T> T getFieldValue(T entity, String fieldName)
            throws IllegalArgumentException, IllegalAccessException {
        Field field = getDeclaredField(entity, fieldName);
        if (null != field)
            return getFieldValue(entity, field);

        return null;
    }

    private static <T> T getFieldValueByCascade(T entity, Field field)
            throws IllegalArgumentException, IllegalAccessException {
        Cascade cascade = field.getAnnotation(Cascade.class);
        if (null != cascade) {
            String mappedBy = cascade.mappedBy();
            String property = cascade.property();

            T mappedObj = getFieldValue(entity, mappedBy);
            if (null != mappedObj)
                return getFieldValue(mappedObj, property);
        }

        return null;
    }

    @SuppressWarnings("unchecked")
    private static <T> T getFieldValueByMethod(T entity, String fieldName)
            throws IllegalArgumentException, IllegalAccessException {
        Object value = null;
        String methodName = StringUtils.EMPTY;
        try {
            methodName = getGetterMethodName(fieldName);
            Method method = entity.getClass().getDeclaredMethod(methodName);
            method.setAccessible(true);
            try {
                value = method.invoke(entity);
            } catch (InvocationTargetException e1) {
                logger.debug(e1);
            }
        } catch (NoSuchMethodException e1) {
            logger.debug("Method cound not find:" + methodName);
        }

        return (T) value;
    }

    private static <T> void setFieldValue(T entity, String fieldName,
                                          Object value, String... formatters)
            throws IllegalArgumentException, IllegalAccessException,
            ParseException {
        Field field = getDeclaredField(entity, fieldName);
        if (null != field) {
            setFieldValue(entity, field, value, formatters);
        }
    }

    private static <T> void setFieldValue(T entity, Field field, Object value,
                                          String... formatters) throws IllegalArgumentException,
            IllegalAccessException, ParseException {
        field.setAccessible(true);

        if (field.getType() == Date.class || field.getType() == Timestamp.class) {
            String dataFormatter = ((null == formatters
                    || 0 >= formatters.length || isNullOrEmpty(formatters[0])) ? "yyyy-MM-dd HH:mm:ss"
                    : formatters[0]).toString();
            SimpleDateFormat sdf = new SimpleDateFormat(dataFormatter);
            if (null != value && value.getClass() == String.class
                    && !StringUtils.isEmpty(value.toString())) {
                if (value.toString().length() != dataFormatter.length()) {
                    String fillValue = dataFormatter.substring(
                            value.toString().length()).replaceAll("[A-Za-z]",
                            "0");
                    value = value.toString() + fillValue;
                }

                if (field.getType() == Timestamp.class)
                    field.set(entity, Timestamp.valueOf(value.toString()));
                else
                    field.set(entity, sdf.parse(value.toString()));
            } else if (null != value
                    && (value.getClass() == Date.class || value.getClass()
                    .getSuperclass() == Date.class)) {
                field.set(entity, value);
            } else {
                field.set(entity, null);
            }
        } else if (field.getType() == Long.class) {
            if (!isNullOrEmpty(value)) {
                field.set(entity, Long.valueOf(value.toString()));
            } else {
                field.set(entity, null);
            }
        } else if (field.getType() == Integer.class
                || field.getType() == int.class) {
            if (!isNullOrEmpty(value)) {
                field.set(entity, new BigDecimal(value.toString()).intValue());
            } else {
                field.set(entity, null);
            }
        } else if (field.getType() == boolean.class) {
            if (!isNullOrEmpty(value)) {
                field.set(entity, Boolean.valueOf(value.toString()));
            } else {
                field.set(entity, false);
            }
        } else if (field.getType() == BigDecimal.class) {
            if (!isNullOrEmpty(value)) {
                field.set(entity,
                        new BigDecimal(value.toString().replace(",", "")));
            } else {
                field.set(entity, null);
            }
        } else if (field.getType() == Short.class) {
            if (!isNullOrEmpty(value)) {
                field.set(entity, Short.valueOf(value.toString()));
            } else {
                field.set(entity, null);
            }
        } else {
            field.set(entity, isNullOrEmpty(value) ? null : value);
        }
    }

    private static <T> void setParentEntity(T entity, Object childObj)
            throws IllegalArgumentException, IllegalAccessException,
            ParseException {
        Field[] fields = childObj.getClass().getDeclaredFields();
        if (null != fields && 0 < fields.length) {
            for (Field field : fields) {
                field.setAccessible(true);
                if (field.getType() == entity.getClass()) {
                    setFieldValue(childObj, field, entity);
                    // field.set(childObj, entity);
                }
            }
        }
    }

    private static boolean isNullOrEmpty(Object value) {
        return null == value || StringUtils.isEmpty(value.toString())
                || value.toString().toUpperCase().equals("NULL");
    }

    private static boolean isNotClone(Field field) {
        NotClone notClone = field.getAnnotation(NotClone.class);
        return null != notClone;
    }

    private static boolean isNotCopy(Field field) {
        NotCopy notCopy = field.getAnnotation(NotCopy.class);
        return null != notCopy;
    }

    private static boolean isIgnore(Field field) {
        if (null == field)
            return false;
        Ignore ignore = field.getAnnotation(Ignore.class);

        return null != ignore;
    }

    private static boolean isIgnore(Field field, String[] ignoreFields) {
        if (null == ignoreFields || 0 >= ignoreFields.length)
            return false;

        for (String fieldName : ignoreFields) {
            if (field.getName().equals(fieldName))
                return true;
        }

        return isIgnore(field);
    }

    private static boolean isNotMapped(Field field) {
        if (null == field)
            return false;
        NotMapped notmapped = field.getAnnotation(NotMapped.class);

        return null != notmapped;
    }

    private static boolean isNotCompare(Field field) {
        if (null == field)
            return false;
        NotCompare compare = field.getAnnotation(NotCompare.class);

        return null != compare;
    }

    private static boolean isTransient(Field field) {
        if (null == field)
            return false;

        return field.getModifiers() == Modifier.PRIVATE + Modifier.TRANSIENT;
    }

    private static boolean hasCascade(Field field) {
        Cascade cascade = field.getAnnotation(Cascade.class);
        return null != cascade;
    }
}
