package com.bettersoft.api.utils;

import io.jsonwebtoken.Claims;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

import java.security.Key;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.crypto.spec.SecretKeySpec;

import com.bettersoft.api.exceptions.ValidateAuthorityFailedException;
import com.bettersoft.api.exceptions.ValidateExpirationFailedException;

public class JwtUtils {
    private static final long EXP = 60 * 60 * 1000; // 有效期为60分钟

    /**
     * 生成token，默认60分钟
     *
     * @param payload 载荷
     * @return token
     * @throws Exception
     */
    public static String createToken(Map<String, Object> payload)
            throws Exception { // 默认有效期为60分钟
        return createToken(payload, EXP);
    }

    /**
     * 生成token
     *
     * @param payload 载荷
     * @return token
     * @throws Exception
     */
    public static String createToken(Object payload) throws Exception {
        Map<String, Object> payloadMap = EntityUtil.convertEntityToMap(payload);

        return createToken(payloadMap, EXP);
    }

    public static String createToken(Object payload, long exp) throws Exception {
        Map<String, Object> payloadMap = EntityUtil.convertEntityToMap(payload);

        return createToken(payloadMap, exp);
    }

    /**
     * 生成token
     *
     * @param payload 载荷
     * @param exp     有效时长
     * @return token
     * @throws Exception
     */
    public static String createToken(Map<String, Object> payload, long exp)
            throws Exception {
        // 签名算法使用SHA256算法加密
        SignatureAlgorithm signatureAlgorithm = SignatureAlgorithm.HS256;
        // 加密JWT
        Key signingKey = new SecretKeySpec(
                StringDigestUtil.encodeBase64Hex(Constants.SECRET_KEY
                        .getBytes(Constants.CHARACTERSET_CODING)),
                signatureAlgorithm.getJcaName());
        // 设置JWT声明格式，生成JWT
        JwtBuilder jwtBuilder = Jwts.builder().setHeaderParam("typ", "jwt")
                .setHeaderParam("alg", "HS256").setClaims(payload)
                .setExpiration(new Date(System.currentTimeMillis() + exp)) // token有效期
                .signWith(signatureAlgorithm, signingKey); // 签名算法及签名密钥，将header与payload加密拼接后形成JWT
        // 返回JWT
        return jwtBuilder.compact();
    }

    /**
     * 解析token信息
     *
     * @param token JWT信息
     * @return payload
     */
    public static Claims parseToken(String token) {
        try {
            return Jwts
                    .parser()
                    .setSigningKey(
                            StringDigestUtil.encodeBase64Hex(Constants.SECRET_KEY
                                    .getBytes(Constants.CHARACTERSET_CODING)))
                    .parseClaimsJws(token).getBody();
        } catch (Exception e) {
            if (e instanceof ExpiredJwtException) {
                return ((ExpiredJwtException) e).getClaims();
            }

            return null;
        }
    }

    /**
     * 解析token信息
     *
     * @param token JWT信息
     * @return payload
     */
    public static Claims verifyToken(String token, RedisCacheUtil redisCacheUtil)
            throws Exception {
        if (null != redisCacheUtil.getCache(Constants.LOGIN_TOKEN_KEY + token)) {
            LoginUserInfo loginUser = redisCacheUtil.getCache(Constants.LOGIN_TOKEN_KEY + token);

            Claims claims = parseToken(loginUser.getTicket());
            if (null == claims)
                throw new ValidateAuthorityFailedException();

            if (0 > claims.getExpiration().compareTo(new Date()))
                throw new ValidateExpirationFailedException();

            return claims;
        }

        return null;
    }
}
