package com.bettersoft.api.utils;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.TimeUnit;

import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.RedisTemplate;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

public class RedisCacheUtil {
	private RedisTemplate<String, Object> redisTemplate;
	// private StringRedisTemplate stringRedisTemplate;

	protected Gson gson = new GsonBuilder().serializeNulls()
			.registerTypeAdapterFactory(HibernateProxyTypeAdapter.FACTORY)
			// .registerTypeAdapter(Map.class, new MapTypeAdapter())
			.registerTypeAdapter(BigDecimal.class, new BigDecimalAdapter())
			.setDateFormat("yyyy-MM-dd HH:mm:ss").create();

	public void setRedisTemplate(RedisTemplate<String, Object> redisTemplate) {
		this.redisTemplate = redisTemplate;
	}

	// public void setStringRedisTemplate(StringRedisTemplate
	// stringRedisTemplate) {
	// this.stringRedisTemplate = stringRedisTemplate;
	// }

	/**
	 * key是否存在缓存中
	 *
	 * @param key
	 *            缓存的键值
	 */
	public boolean containsKey(String key) {
		return redisTemplate.hasKey(key);
	}

	/**
	 * 获取缓存基本的对象，Integer、String、实体类等
	 *
	 * @param key
	 *            缓存的键值
	 */
	@SuppressWarnings("unchecked")
	public <T> T getCache(String key) {
		if (!containsKey(key))
			return null;

		return (T) redisTemplate.opsForValue().get(key);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public <T> T getCache(String key, String hashKey) {
		if (!containsKey(key))
			return null;
		HashOperations hashOper = redisTemplate.opsForHash();

		return (T) hashOper.get(key, hashKey);
	}

	/**
	 * 缓存基本的对象，Integer、String、实体类等
	 *
	 * @param key
	 *            缓存的键值
	 * @param value
	 *            缓存的值
	 */
	public <T> void setCache(String key, T value) {
		redisTemplate.opsForValue().set(key, value);
	}

	public <T> void setCache(String key, T value, long timeout) {
		redisTemplate.opsForValue().set(key, value, timeout, TimeUnit.SECONDS);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public <T> void setCache(String key, String hashKey, T value) {
		HashOperations hashOper = redisTemplate.opsForHash();

		hashOper.put(key, hashKey, value);
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public <T> void setCache(String key, String hashKey, T value, long timeout) {
		HashOperations hashOper = redisTemplate.opsForHash();

		hashOper.put(key, hashKey, value);
		redisTemplate.expire(key, timeout, TimeUnit.SECONDS);
	}

	/**
	 * 缓存基本的MAP对像
	 *
	 * @param key
	 *            缓存的键值
	 * @param hashKey
	 *            MAP的键值
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public Map getCacheMap(String key) {
		if (!containsKey(key))
			return null;

		Map resultMap = new HashMap();
		HashOperations hashOper = redisTemplate.opsForHash();
		Iterator iter = hashOper.keys(key).iterator();
		while (iter.hasNext()) {
			Object hashKey = iter.next();
			resultMap.put(hashKey, hashOper.get(key, hashKey));
		}
		return resultMap;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void setCacheMap(String key, Map map) {
		HashOperations hashOper = redisTemplate.opsForHash();
		Iterator<Entry> iter = map.entrySet().iterator();
		while (iter.hasNext()) {
			Entry entry = iter.next();
			hashOper.put(key, entry.getKey(), entry.getValue());
		}
	}

	@SuppressWarnings("unchecked")
	public <T> List<T> getCasheList(String key, boolean... removeAble) {
		List<T> datas = new ArrayList<T>();
		boolean remove = null != removeAble && 0 < removeAble.length ? removeAble[0]
				: false;
		ListOperations<String, T> listOper = (ListOperations<String, T>) redisTemplate
				.opsForList();

		for (int i = 0; i < listOper.size(key); i++) {
			datas.add((T) listOper.index(key, i));

			if (remove)
				listOper.rightPop(key);
		}

		return datas;
	}

	@SuppressWarnings("unchecked")
	public <T> List<T> getCasheHashList(String key) {
		List<T> datas = new ArrayList<T>();

		Iterator<T> iter = (Iterator<T>) redisTemplate.opsForHash()
				.entries(key).values().iterator();
		while (iter.hasNext()) {
			T entity = iter.next();
			datas.add(entity);
		}

		return datas;
	}

	/**
	 * 缓存List数据
	 *
	 * @param key
	 *            缓存的键值
	 * @param datas
	 *            待缓存的List数据
	 */
	@SuppressWarnings({ "unchecked" })
	public <T> void setCacheList(String key, List<T> datas) {
		ListOperations<String, T> listOper = (ListOperations<String, T>) redisTemplate
				.opsForList();
		if (null != datas && !datas.isEmpty()) {
			for (T d : datas) {
				listOper.rightPush(key, d);
			}
		}
	}

	public void removeCacheByKey(String key) {
		if (containsKey(key)) {
			redisTemplate.delete(key);
		}
	}

	public void removeCacheByKey(String key, String hashKey) {
		if (containsKey(key)) {
			redisTemplate.opsForHash().delete(key, hashKey);
		}
	}
}
